#include "box.h"

Box::Box(const Vector& llf,const Vector& urb) : llf_(llf), urb_(urb) {    
}


const Vector& Box::getLLF() const { return llf_; }

void Box::setLLF(const Vector& v) { llf_ = v; }

const Vector& Box::getURB() const { return urb_; }

void Box::setURB(const Vector& n) { urb_ = n; }

inline double max(double x, double y) {
	return (x<y) ? y : x;
}
inline double min(double x, double y) {
	return (x>y) ? y : x;
}

bool Box::intersect(const Ray& ray,Vector& point,Vector& normal, double& distance) const {
	const static Vector NormalRight(+1,0,0);
	const static Vector NormalLeft (-1,0,0);
	const static Vector NormalUp   (0,+1,0);
	const static Vector NormalDown (0,-1,0);
	const static Vector NormalFront(0,0,+1);
	const static Vector NormalBack (0,0,-1);

	// ray origin
	const Vector origin = ray.getOrigin();
	// ray direction
	const Vector direction = ray.getDirection();


	// ray enters box (-infinity)
	double tnear = -999999999999;
	// ray leaves box (+infinity)
	double tfar  = +999999999999;


	/////////////////////////////////////////////////////////
	// x
	const double& ox = origin[0];
	const double& rx = direction[0];

	// parallel to box ?
	if (rx == 0)
		if (ox < llf_[0] || ox > urb_[0])
			return false;

	// intersections
	const double tx1 = (llf_[0] - ox) / rx;
	const double tx2 = (urb_[0] - ox) / rx;

	// adjust tnear and tfar
	if (tx2 > tx1 && tx1 > tnear)
	{
		tnear  = tx1;
		normal = NormalLeft;
	}
	if (tx1 > tx2 && tx2 > tnear)
	{
		tnear = tx2;
		normal = NormalRight;
	}

	// ray doesn't hit box ?
	tfar = min(tfar, max(tx1, tx2));
	if (tnear > tfar || tfar < 0)
		return false;
    

	/////////////////////////////////////////////////////////
	// y
	const double& oy = origin[1];
	const double& ry = direction[1];

	// parallel to box ?
	if (ry == 0)
		if (oy < llf_[1] || oy > urb_[1])
			return false;

	// intersections
	const double ty1 = (llf_[1] - oy) / ry;
	const double ty2 = (urb_[1] - oy) / ry;

	// adjust tnear and tfar
	if (ty2 > ty1 && ty1 > tnear)
	{
		tnear  = ty1;
		normal = NormalDown;
	}
	if (ty1 > ty2 && ty2 > tnear)
	{
		tnear  = ty2;
		normal = NormalUp;
	}

	// ray doesn't hit box ?
	tfar = min(tfar, max(ty1, ty2));
	if (tnear > tfar || tfar < 0)
		return false;


	/////////////////////////////////////////////////////////
	// z
	const double& oz = origin[2];
	const double& rz = direction[2];

	// parallel to box ?
	if (rz == 0)
		if (oz < llf_[2] || oz > urb_[2])
			return false;

	// intersections
	const double tz1 = (llf_[2] - oz) / rz;
	const double tz2 = (urb_[2] - oz) / rz;

	// adjust tnear and tfar
	if (tz2 > tz1 && tz1 > tnear)
	{
		tnear  = tz1;
		normal = NormalBack;
	}
	if (tz1 > tz2 && tz2 > tnear)
	{
		tnear  = tz2;
		normal = NormalFront;
	}

	// ray doesn't hit box ?
	tfar = min(tfar, max(tz1, tz2));
	if (tnear > tfar || tfar < 0)
		return false;


	// intersection finally found !!!
	point  = ray.getOrigin() + tnear*ray.getDirection();
	distance = tnear;//abs(point-ray.getOrigin());

	// done !
	return true;
}