#include "triangle.h"
#include <iostream.h>
#include <assert.h>
#include <algorithm>

Triangle::Triangle() { }

Triangle::Triangle(const Vector& v0,const Vector& v1, const Vector& v2) {
    v_[0] = v0; 
    v_[1] = v1; 
    v_[2] = v2;
    triNrm_ = makeCWTriangleNormal(v_[0],v_[1],v_[2]);
    n_[0] = n_[1] = n_[2] = triNrm_;
}


void Triangle::normalizeNormals() {
    n_[0].normalize();
    n_[1].normalize();
    n_[2].normalize();
}

static inline int MeshMaxIndex(double a, double b, double c) {
    return((a > b) ? ((a > c) ? 0 : 2) : ((b > c) ? 1 : 2));
}

bool Triangle::intersect(const Ray& ray,Vector& point,Vector& normal, double& distance) const {
	// edges of the triangle
	const Vector edge1 = v_[1] - v_[0];
	const Vector edge2 = v_[2] - v_[0];

	const Vector p    = ray.getDirection() * edge2;
	const double detA = dotProduct(p, edge1);

	// det(A) != 0
	if (detA == 0)
		return false;

	const double f = 1/detA;
	const Vector s = ray.getOrigin() - v_[0];
	const double u = f * dotProduct(p, s);

	// first barycentric value must be in [0,1]
	if (u<0 || u>1)
		return false;

	const Vector q = s*edge1;
	const double v = f * dotProduct(q, ray.getDirection());

	// second barycentric value must be in [0,1]
	if (v<0 || v>1 || u+v>1)
		return false;

	const double t = f * dotProduct(q, edge2);

	// object on the wrong side of the ray ?
	if (t <= 0)
		return false;


	// intersection found, barycentric coordinates are (u,v,t)
	point  = ray.getOrigin() + t*ray.getDirection();

	normal = triNrm_;
	// invert normal to allow the triangle being viewed from its back side
	if (dotProduct(normal, ray.getDirection()) > 0)
		normal = -normal;

	distance = t;//abs(point-ray.getOrigin());

    return true;
}

//
// IO
//

istream& operator>>(istream& s, Triangle& u) {    
    Vector t;
    for (int i = 0; i < 3; i++) {
        if (!(s >> t[0] >> t[1] >> t[2])) {
            assert(0);
        }
        u.v_[i] = t;
    }            
    u.triNrm_ = makeCWTriangleNormal(u.v_[0],u.v_[1],u.v_[2]);
    u.n_[0] = u.n_[1] = u.n_[2] = u.triNrm_;
    return s;
}