#ifndef _UTIL_H
#define _UTIL_H

#include "vector.h"
#include "triangle.h"
#include "box.h"
#include "sphere.h"
#include <fstream.h>


// Color class 
typedef Vector Color;


// Camera class
class Camera {
public:
    Camera(const Vector& f, const Vector& t, const Vector& u, double fo) 
        : from(f), to(t), up(u.normalized()), fovy(fo) {}
    Vector from; 
    Vector to; 
    Vector up; 
    double fovy;
};

// Light class
class Light {
public:    
    Light(const Vector& p, double i, double a) 
        : pos(p),intensity(i),ambient(a),att_constant(1), att_linear(0), att_quadric(0) {}
    
    Vector pos;                  /* position, world coordinate system */
    double intensity;            /* light source intensity [0,1]      */
    double ambient;              /* ambient light contribution        */
	double att_constant;		 /* attenuation factors               */
	double att_linear;
	double att_quadric;
};


// Material class
class Material {
public:
    Material(double ka_,double kd_,double ks_,double n_, double opacity_, double lightspeed_, Color c) 
        : ka(ka_), kd(kd_), ks(ks_), n(n_), opacity(opacity_), lightspeed(lightspeed_), color(c) {} 
    
    double ka;                   /* ambient reflection coefficient  */
    double kd;                   /* diffuse reflection coefficient  */
    double ks;                   /* specular reflection coefficient */
    double n;                    /* specular shininess              */
	double opacity;				 /* opacity, 1=no refraction        */
	double lightspeed;			 /* speed of light, used for refraction */
    Color color;                 /* specular color                  */    
};


#endif // _UTIL_H
